@@ -24,7 +24,6 @@
 # IN THE SOFTWARE.
 
 import fnmatch
-import locale
 import logging
 import multiprocessing
 import os
@@ -39,6 +38,7 @@ from urllib.parse import quote as url_quote
 
 from click import get_terminal_size, progressbar
 from natsort import natsort_keygen, ns
+from PIL import Image as PILImage
 
 from . import image, signals, video
 from .image import get_exif_tags, get_image_metadata, get_size, process_image
@@ -56,7 +56,7 @@ class Media:
     Attributes:
 
     :var Media.type: ``"image"`` or ``"video"``.
-    :var Media.filename: Filename of the resized image.
+    :var Media.dst_filename: Filename of the resized image.
     :var Media.thumbnail: Location of the corresponding thumbnail image.
     :var Media.big: If not None, location of the unmodified image.
     :var Media.big_url: If not None, url of the unmodified image.
@@ -67,21 +67,23 @@ class Media:
     """Type of media, e.g. ``"image"`` or ``"video"``."""
 
     def __init__(self, filename, path, settings):
-        self.filename = filename
+        self.path = path
+        self.settings = settings
+
+        self.basename = os.path.splitext(filename)[0]
+
+        self.dst_filename = filename
         """Filename of the resized image."""
 
         self.src_filename = filename
-        """Filename of the resized image."""
+        """Filename of the input image."""
 
-        self.path = path
-        self.settings = settings
-        self.ext = os.path.splitext(filename)[1].lower()
+        self.src_ext = os.path.splitext(filename)[1].lower()
+        """Input extension."""
 
-        self.src_path = join(settings['source'], path, filename)
-        self.dst_path = join(settings['destination'], path, filename)
+        self.src_path = join(settings['source'], path, self.src_filename)
 
-        self.thumb_name = get_thumb(self.settings, self.filename)
-        self.thumb_path = join(settings['destination'], path, self.thumb_name)
+        self.thumb_name = get_thumb(self.settings, self.dst_filename)
 
         self.logger = logging.getLogger(__name__)
 
@@ -90,19 +92,38 @@ class Media:
 
         # default: title is the filename
         if not self.title:
-            self.title = self.filename
+            self.title = self.basename
         signals.media_initialized.send(self)
 
     def __repr__(self):
         return "<{}>({!r})".format(self.__class__.__name__, str(self))
 
     def __str__(self):
-        return join(self.path, self.filename)
+        return join(self.path, self.src_filename)
+
+    def __getstate__(self):
+        state = self.__dict__.copy()
+        # remove un-pickable objects
+        state['logger'] = None
+        return state
+
+    def __setstate__(self, state):
+        for slot, value in state.items():
+            setattr(self, slot, value)
+        self.logger = logging.getLogger(__name__)
+
+    @property
+    def dst_path(self):
+        return join(self.settings['destination'], self.path, self.dst_filename)
+
+    @property
+    def thumb_path(self):
+        return join(self.settings['destination'], self.path, self.thumb_name)
 
     @property
     def url(self):
         """URL of the media."""
-        return url_from_path(self.filename)
+        return url_from_path(self.dst_filename)
 
     @property
     def big(self):
@@ -113,7 +134,7 @@ class Media:
             s = self.settings
             if s['use_orig']:
                 # The image *is* the original, just use it
-                return self.filename
+                return self.src_filename
             orig_path = join(s['destination'], self.path, s['orig_dir'])
             check_or_create_dir(orig_path)
             big_path = join(orig_path, self.src_filename)
@@ -181,6 +202,17 @@ class Image(Media):
 
     type = 'image'
 
+    def __init__(self, filename, path, settings):
+        super().__init__(filename, path, settings)
+        imgformat = settings.get('img_format')
+
+        if imgformat and PILImage.EXTENSION[self.src_ext] != imgformat.upper():
+            # Find the extension that should match img_format
+            extensions = {v: k for k, v in PILImage.EXTENSION.items()}
+            ext = extensions[imgformat.upper()]
+            self.dst_filename = self.basename + ext
+            self.thumb_name = get_thumb(self.settings, self.dst_filename)
+
     @cached_property
     def date(self):
         """The date from the EXIF DateTimeOriginal metadata if available, or
@@ -195,7 +227,7 @@ class Image(Media):
         """
         datetime_format = self.settings['datetime_format']
         return (get_exif_tags(self.raw_exif, datetime_format=datetime_format)
-                if self.raw_exif and self.ext in ('.jpg', '.jpeg') else None)
+                if self.raw_exif and self.src_ext in ('.jpg', '.jpeg') else None)
 
     def _get_metadata(self):
         super()._get_metadata()
@@ -216,7 +248,7 @@ class Image(Media):
     @cached_property
     def raw_exif(self):
         """If not `None`, contains the raw EXIF tags."""
-        if self.ext in ('.jpg', '.jpeg'):
+        if self.src_ext in ('.jpg', '.jpeg'):
             return self.file_metadata['exif']
 
     @cached_property
@@ -224,6 +256,11 @@ class Image(Media):
         """The dimensions of the resized image."""
         return get_size(self.dst_path)
 
+    @cached_property
+    def input_size(self):
+        """The dimensions of the input image."""
+        return get_size(self.src_path)
+
     @cached_property
     def thumb_size(self):
         """The dimensions of the thumbnail image."""
@@ -241,17 +278,15 @@ class Video(Media):
 
     def __init__(self, filename, path, settings):
         super().__init__(filename, path, settings)
-        base, ext = splitext(filename)
-        self.src_filename = filename
         self.date = self._get_file_date()
-        if not settings['use_orig'] or not is_valid_html5_video(ext):
+
+        if not settings['use_orig'] or not is_valid_html5_video(self.src_ext):
             video_format = settings['video_format']
             ext = '.' + video_format
-            self.filename = base + ext
+            self.dst_filename = self.basename + ext
             self.mime = get_mime(ext)
-            self.dst_path = join(settings['destination'], path, base + ext)
         else:
-            self.mime = get_mime(ext)
+            self.mime = get_mime(self.src_ext)
 
 
 class Album:
@@ -369,36 +404,52 @@ class Album:
 
     def sort_subdirs(self, albums_sort_attr):
         if self.subdirs:
-            if albums_sort_attr:
-                root_path = self.path if self.path != '.' else ''
-                if albums_sort_attr.startswith("meta."):
-                    meta_key = albums_sort_attr.split(".", 1)[1]
-                    key = natsort_keygen(key=lambda s:
-                        self.gallery.albums[join(root_path, s)].meta.get(meta_key, [''])[0],
-                        alg=ns.LOCALE)
-                else:
-                    key = natsort_keygen(key=lambda s:
-                        getattr(self.gallery.albums[join(root_path, s)], albums_sort_attr),
-                        alg=ns.LOCALE)
-            else:
-                key = natsort_keygen(alg=ns.LOCALE)
+            if not albums_sort_attr:
+                albums_sort_attr = self.settings['albums_sort_attr']
+            reverse = self.settings['albums_sort_reverse']
 
-            self.subdirs.sort(key=key,
-                              reverse=self.settings['albums_sort_reverse'])
+            if 'sort' in self.meta:
+                albums_sort_attr = self.meta['sort'][0]
+                if albums_sort_attr[0] == '-':
+                    albums_sort_attr = albums_sort_attr[1:]
+                    reverse = True
+                else:
+                    reverse = False
+
+            root_path = self.path if self.path != '.' else ''
+            if albums_sort_attr.startswith("meta."):
+                meta_key = albums_sort_attr.split(".", 1)[1]
+
+                def sort_key(s):
+                    album = self.gallery.albums[join(root_path, s)]
+                    return album.meta.get(meta_key, [''])[0]
+
+            else:
+                def sort_key(s):
+                    album = self.gallery.albums[join(root_path, s)]
+                    return getattr(album, albums_sort_attr)
+
+            key = natsort_keygen(key=sort_key, alg=ns.LOCALE)
+            self.subdirs.sort(key=key, reverse=reverse)
 
         signals.albums_sorted.send(self)
 
     def sort_medias(self, medias_sort_attr):
         if self.medias:
+            if medias_sort_attr == 'filename':
+                medias_sort_attr = 'dst_filename'
+
             if medias_sort_attr == 'date':
                 key = lambda s: s.date or datetime.now()
             elif medias_sort_attr.startswith('meta.'):
                 meta_key = medias_sort_attr.split(".", 1)[1]
-                key = natsort_keygen(key=lambda s: s.meta.get(meta_key, [''])[0],
-                                    alg=ns.LOCALE)
+                key = natsort_keygen(
+                    key=lambda s: s.meta.get(meta_key, [''])[0],
+                    alg=ns.LOCALE)
             else:
-                key = natsort_keygen(key=lambda s: getattr(s, medias_sort_attr),
-                                    alg=ns.LOCALE)
+                key = natsort_keygen(
+                    key=lambda s: getattr(s, medias_sort_attr),
+                    alg=ns.LOCALE)
 
             self.medias.sort(key=key,
                              reverse=self.settings['medias_sort_reverse'])
@@ -428,6 +479,10 @@ class Album:
         return [self.gallery.albums[join(root_path, path)]
                 for path in self.subdirs]
 
+    @property
+    def nbmedias(self):
+        return len(self.medias) + sum(len(album) for album in self.albums)
+
     @property
     def url(self):
         """URL of the album, relative to its parent."""
@@ -453,17 +508,24 @@ class Album:
         else:
             # find and return the first landscape image
             for f in self.medias:
-                ext = splitext(f.filename)[1]
-                if ext.lower() in self.settings['img_extensions']:
-                    # Use f.size if available as it is quicker (in cache), but
-                    # fallback to the size of src_path if dst_path is missing
-                    size = f.size
-                    if size is None:
-                        size = f.file_metadata['size']
+                ext = splitext(f.dst_filename)[1]
+                if ext.lower() not in self.settings['img_extensions']:
+                    continue
 
-                    if size['width'] > size['height']:
+                # Use f.size if available as it is quicker (in cache), but
+                # fallback to the size of src_path if dst_path is missing
+                size = f.input_size
+                if size is None:
+                    size = f.file_metadata['size']
+
+                if size['width'] > size['height']:
+                    try:
                         self._thumbnail = (url_quote(self.name) + '/' +
                                            f.thumbnail)
+                    except Exception as e:
+                        self.logger.info("Failed to get thumbnail for %s: %s",
+                                         f.dst_filename, e)
+                    else:
                         self.logger.debug(
                             "Use 1st landscape image as thumbnail for %r : %s",
                             self, self._thumbnail)
@@ -473,12 +535,19 @@ class Album:
             if not self._thumbnail and self.medias:
                 for media in self.medias:
                     if media.thumbnail is not None:
-                        self._thumbnail = (url_quote(self.name) + '/' +
-                                           media.thumbnail)
-                        break
+                        try:
+                            self._thumbnail = (url_quote(self.name) + '/' +
+                                               media.thumbnail)
+                        except Exception as e:
+                            self.logger.info(
+                                "Failed to get thumbnail for %s: %s",
+                                media.dst_filename, e
+                            )
+                        else:
+                            break
                 else:
                     self.logger.warning("No thumbnail found for %r", self)
-                    return None
+                    return
 
                 self.logger.debug("Use the 1st image as thumbnail for %r : %s",
                                   self, self._thumbnail)
@@ -496,7 +565,6 @@ class Album:
                         return self._thumbnail
 
         self.logger.error('Thumbnail not found for %r', self)
-        return None
 
     @property
     def random_thumbnail(self):
@@ -540,7 +608,6 @@ class Album:
         """Placeholder ZIP method.
         The ZIP logic is controlled by the zip_gallery plugin
         """
-        return None
 
 
 class Gallery:
@@ -681,14 +748,13 @@ class Gallery:
         bar_opt = {'label': "Processing files",
                    'show_pos': True,
                    'file': self.progressbar_target}
-        failed_files = []
 
         if self.pool:
+            result = []
             try:
                 with progressbar(length=len(media_list), **bar_opt) as bar:
-                    for res in self.pool.imap_unordered(worker, media_list):
-                        if res:
-                            failed_files.append(res)
+                    for status in self.pool.imap_unordered(worker, media_list):
+                        result.append(status)
                         bar.update(1)
                 self.pool.close()
                 self.pool.join()
@@ -704,12 +770,11 @@ class Gallery:
                 sys.exit('Abort')
         else:
             with progressbar(media_list, **bar_opt) as medias:
-                for media_item in medias:
-                    res = process_file(media_item)
-                    if res:
-                        failed_files.append(res)
+                result = [process_file(media_item) for media_item in medias]
 
-        if failed_files:
+        if any(result):
+            failed_files = [media for status, media in zip(result, media_list)
+                            if status != 0]
             self.remove_files(failed_files)
 
         if self.settings['write_html']:
@@ -737,13 +802,13 @@ class Gallery:
 
         signals.gallery_build.send(self)
 
-    def remove_files(self, files):
+    def remove_files(self, medias):
         self.logger.error('Some files have failed to be processed:')
-        for path, filename in files:
-            self.logger.error('  - %s/%s', path, filename)
-            album = self.albums[path]
+        for media in medias:
+            self.logger.error('  - %s', media.dst_filename)
+            album = self.albums[media.path]
             for f in album.medias:
-                if f.filename == filename:
+                if f.dst_filename == media.dst_filename:
                     self.stats[f.type + '_failed'] += 1
                     album.medias.remove(f)
                     break
@@ -754,21 +819,16 @@ class Gallery:
         """Process a list of images in a directory."""
         for f in album:
             if isfile(f.dst_path) and not force:
-                self.logger.info("%s exists - skipping", f.filename)
+                self.logger.info("%s exists - skipping", f.dst_filename)
                 self.stats[f.type + '_skipped'] += 1
             else:
                 self.stats[f.type] += 1
-                yield (f.type, f.path, f.filename, f.src_path, album.dst_path,
-                       self.settings)
+                yield f
 
 
-def process_file(args):
-    # args => ftype, path, filename, src_path, dst_path, settings
-    processor = process_image if args[0] == 'image' else process_video
-    ret = processor(*args[3:])
-    # If the processor return an error (ret != 0), then we return the path and
-    # filename of the failed file to the parent process.
-    return args[1:3] if ret else None
+def process_file(media):
+    processor = process_image if media.type == 'image' else process_video
+    return processor(media)
 
 
 def worker(args):
