@@ -67,17 +67,14 @@ def _read_image(file_path):
     if isinstance(file_path, PILImage.Image):
         return file_path
 
+    logger = logging.getLogger(__name__)
+
     with warnings.catch_warnings(record=True) as caught_warnings:
         im = PILImage.open(file_path)
 
-    for warning in caught_warnings:
-        if warning.category == PILImage.DecompressionBombWarning:
-            logger = logging.getLogger(__name__)
-            logger.info('PILImage reported a possible DecompressionBomb'
-                        ' for file {}'.format(file_path))
-        else:
-            warnings.showwarning(warning.message, warning.category,
-                                 warning.filename, warning.lineno)
+    for w in caught_warnings:
+        logger.warning(f'PILImage reported a warning for file {file_path}\n'
+                       f'{w.category}: {w.message}')
     return im
 
 
@@ -146,7 +143,9 @@ def generate_image(source, outname, settings, options=None):
 
     # first, use hard-coded output format, or PIL format, or original image
     # format, or fall back to JPEG
-    outformat = settings.get('img_format') or img.format or original_format or 'JPEG'
+    outformat = (settings.get('img_format') or img.format or
+                 original_format or 'JPEG')
+
     logger.debug('Save resized image to %s (%s)', outname, outformat)
     save_image(img, outname, outformat, options=options, autoconvert=True)
 
@@ -157,6 +156,7 @@ def generate_thumbnail(source, outname, box, fit=True, options=None,
 
     logger = logging.getLogger(__name__)
     img = _read_image(source)
+    img = Transpose().process(img)
     original_format = img.format
 
     if fit:
@@ -170,31 +170,32 @@ def generate_thumbnail(source, outname, box, fit=True, options=None,
     save_image(img, outname, outformat, options=options, autoconvert=True)
 
 
-def process_image(filepath, outpath, settings):
+def process_image(media):
     """Process one image: resize, create thumbnail."""
 
     logger = logging.getLogger(__name__)
-    logger.info('Processing %s', filepath)
-    filename = os.path.split(filepath)[1]
-    outname = os.path.join(outpath, filename)
-    ext = os.path.splitext(filename)[1]
+    logger.info('Processing %s', media.src_path)
 
-    if ext in ('.jpg', '.jpeg', '.JPG', '.JPEG'):
-        options = settings['jpg_options']
-    elif ext == '.png':
+    if media.src_ext in ('.jpg', '.jpeg', '.JPG', '.JPEG'):
+        options = media.settings['jpg_options']
+    elif media.src_ext == '.png':
         options = {'optimize': True}
     else:
         options = {}
 
     try:
-        generate_image(filepath, outname, settings, options=options)
+        generate_image(media.src_path, media.dst_path, media.settings,
+                       options=options)
 
-        if settings['make_thumbs']:
-            thumb_name = os.path.join(outpath, get_thumb(settings, filename))
+        if media.settings['make_thumbs']:
             generate_thumbnail(
-                outname, thumb_name, settings['thumb_size'],
-                fit=settings['thumb_fit'], options=options,
-                thumb_fit_centering=settings["thumb_fit_centering"])
+                media.dst_path,
+                media.thumb_path,
+                media.settings['thumb_size'],
+                fit=media.settings['thumb_fit'],
+                options=options,
+                thumb_fit_centering=media.settings["thumb_fit_centering"]
+            )
     except Exception as e:
         logger.info('Failed to process: %r', e)
         if logger.getEffectiveLevel() == logging.DEBUG:
@@ -225,11 +226,18 @@ def get_exif_data(filename):
     img = _read_image(filename)
 
     try:
-        exif = img._getexif() or {}
+        with warnings.catch_warnings(record=True) as caught_warnings:
+            exif = img._getexif() or {}
     except ZeroDivisionError:
         logger.warning('Failed to read EXIF data.')
         return None
 
+    for w in caught_warnings:
+        fname = (filename.filename if isinstance(filename, PILImage.Image)
+                 else filename)
+        logger.warning(f'PILImage reported a warning for file {fname}\n'
+                       f'{w.category}: {w.message}')
+
     data = {TAGS.get(tag, tag): value for tag, value in exif.items()}
 
     if 'GPSInfo' in data:
@@ -237,7 +245,6 @@ def get_exif_data(filename):
             data['GPSInfo'] = {GPSTAGS.get(tag, tag): value
                                for tag, value in data['GPSInfo'].items()}
         except AttributeError:
-            logger = logging.getLogger(__name__)
             logger.info('Failed to get GPS Info')
             del data['GPSInfo']
     return data
